"""Working with the Product data over the REST Web Services:  CRUD operations, Objects, and Fragments

Generic Data Service introduces an approach to work with the Product data over the REST Web Services.
The Service delivers the basic methods to be able to fulfill any operation with the data like CRUD-operations (Create-Read-Update-Delete).
The methods are generic, which means, it fully relies on the state of the Product Schema and allows to work with any Configuration item or
Data Definition presented in the Product Schema. That approach allows extending the Schema with new entities, or customize the present
ones, and then immediately dealing with new artifacts via Web Services.
"""
